declare const HUE_STRUCTURES: {
    none: never[];
    polar: number[];
    split: number[];
    trine: number[];
    square: number[];
};
declare const CHANNEL_SPECIFIC_LUM: {
    R: number;
    G: number;
    B: number;
};

declare const unfiltered: Filter;
declare const CMYK: Filter;

declare const UI: {
    state: {
        hover: {};
        active: {};
        disabled: {};
    };
    bg: {
        shade: number;
        state: {
            base: {};
            hover: {
                shade: number;
            };
            active: {
                shade: number;
            };
            disabled: {
                shade: number;
            };
        };
    };
    mg: {
        shade: number;
    };
    fg: {
        contrast: string;
        state: {
            hover: {
                contrast: string;
            };
            active: {
                contrast: string;
            };
            disabled: {
                shade: number;
            };
        };
    };
};
declare const trifactory: {
    PaletteModule: {
        contextColor: ({
            resetColor: string;
            shade?: never;
        } | {
            shade: number;
            resetColor?: never;
        })[];
        variables: {
            bg: never[];
            mg: ({
                resetColor: number;
                shade?: never;
            } | {
                shade: number;
                resetColor?: never;
            })[];
            fg: {
                contrast: string;
            }[];
        };
        children: {
            Control: {
                states: {
                    base: {};
                    hover: {
                        tint: number;
                    }[];
                    active: {
                        shade: number;
                    }[];
                };
                variables: {
                    bg: {
                        shade: number;
                    }[];
                    mg: {
                        shade: number;
                    }[];
                    fg: {
                        states: {
                            base: ({
                                shade: number;
                                contrast?: never;
                            } | {
                                contrast: string;
                                shade?: never;
                            })[];
                            hover: ({
                                shade: number;
                                contrast?: never;
                            } | {
                                contrast: string;
                                shade?: never;
                            })[];
                            active: ({
                                shade: number;
                                contrast?: never;
                            } | {
                                contrast: string;
                                shade?: never;
                            })[];
                        };
                    };
                };
            };
        };
    };
};
declare const palettes: {
    id: string;
    hex: string;
    states: {
        id: string;
        hex: string;
        elements: {
            id: string;
            hex: string;
        }[];
    }[];
}[];

declare const specToHex: ({ hue, sat, lum, prefer }: LuumSpec, filter?: Filter) => Hex;

type SpecToHexFixLimit = (spec: LuumSpec, filter?: Filter) => {
    hex: Hex;
    fix: LuumFix;
    limit: LuumLimit;
};
declare const specToHexFixLimit: SpecToHexFixLimit;

declare const identifyHue: (hue: Degree) => string;

declare const channelsToSpec: ({ R, G, B }: ChannelObject) => HSL;

declare const _default$3: (maybeHex: string) => ChannelObject;

declare const hexToSpec: (hex: string) => HSL;

/**
 * Gives us the relative values of the channels,
 * irrespective of the white light beneath them.
 * @param {number} hue - in degrees. Gets safely wrapped around first thing.
 *
 * Digital Red      0 ->   0
 *
 * Sunlight         50 ->  50
 *
 * Digital Yellow   60 ->  60
 *
 * Citron           70 ->  70
 *
 * Turquoise        510 -> 150
 *
 * @const {number} hueReduced - hue 0-359.9 is now a floating point 0-5.999.
 *
 * Digital Red    0 ->   0  ~  0.000
 *
 * Sunlight       50 -> 5/6  ~  0.833
 *
 * Digital Yellow 60 ->   1  ~  1.000
 *
 * Citron         70 -> 7/6  ~  1.167
 *
 * Turquoise      150 -> 5/2  ~  2.500
 *
 * @const {number} hueInteger - from 1-6. Tells us what color region we are in.
 *
 * Digital Red    0.000 -> 0
 *                : red-into-yellow region
 *
 * Sunlight       0.833 -> 0
 *                : red-into-yellow region
 *
 * Digital Yellow 1.000 -> 1
 *                : yellow-into-green region
 *
 * Citron         1.167 -> 1
 *                : yellow-into-green region
 *
 * Turquoise      2.500 -> 2
 *                : green-into-cyan region
 *
 * hueInteger is the 'whole number' piece of hueReduced.
 * FYI, the six color regions are bounded by red, yellow, green, cyan, blue, magenta.
 *
 * @const {number} hueDecimal - tells where we are in this region.
 *
 * Digital Red    0.000 -> 0.000
 *                : at the very beginning
 *
 * Sunlight       0.833 -> 0.833
 *                : near the end
 *
 * Digital Yellow 1.000 -> 0.000
 *                : at the very beginning
 *
 * Citron         1.167 -> 0.167
 *                : near the beginning
 *
 * Turquoise      2.500 -> 0.500
 *                : at the halfway point
 *
 * hueDecimal is the 'fraction' piece of hueReduced.
 * we are going to use this number to determine the value of the in-between channel.
 *
 * @const {number} x - used in primary-secondary transitions like Red into Yellow
 * @const {number} y - used in secondary-primary transitions like Yellow into Green
 * To understand the function of x and y, take the difference between
 *
 * Sunlight       (hue 50),
 * Citron         (hue 70),
 * Digital Yellow (hue 60),
 *
 * as an instructive case. These colors are all basically yellow.
 *
 * Sunlight is hue 50, which puts it near the end of the red-into-yellow region.
 *
 * This means its Red channel is full, and its Green channel is almost full.
 * The fullness of its Green channel is directly proportional to its
 * hueDecimal, the distance from the beginning of this region: 0.833
 *
 * Citron is hue 70, which puts it near the beginning of the yellow-into-green region.
 *
 * This means its Red channel is ALMOST FULL, and its Green channel is FULL.
 * So the fullness of its Red channel is INVERSELY proportional to its
 * hueDecimal, the distance from the beginning of this region: 1 - 0.167 = 0.833
 *
 * Digital Yellow is hue 60, which puts it at the very beginning of the yellow-into-green region.
 *
 * This means its Red Channel and its Green channel must both be full.
 * Like Citron, the fullness of Digital Yellow's Red channel is inversely proportional to its
 * hueDecimal, which is 0. Therefore Digital Yellow's Red channel has a fullness of 1.
 *
 * @returns array of values reflecting the spread between channels
 *
 * Digital Red    case 0:  [   R ===== 1       G = x = 0.000   B ===== 0      ]
 *
 * Sunlight       case 0:  [   R ===== 1       G = x = 0.833   B ===== 0      ]
 *
 * DigitalYellow  case 1:  [   R = y = 1.000   G ===== 1       B ===== 0      ]
 *
 * Citron         case 1:  [   R = y = 0.833   G ===== 1       B ===== 0      ]
 *
 * Turquoise      case 2:  [   R ===== 0       G ===== 1       B = x = 0.500  ]
 *
 * here we see detailed breakdowns of the function's final output for our running examples.
 */
declare const _default$2: (hue: Degree) => [r: Fraction, g: Fraction, b: Fraction];

declare const normalizeHex: (maybeHex: string) => Hex;

declare class Luum implements LuumSpec {
    hue: number;
    sat: number;
    lum: number;
    prefer: `lum` | `sat`;
    constructor(json?: Partial<LuumSpec>);
    toJSON(): LuumSpec;
    static fromJSON(json: unknown): Luum;
    toHex(): string;
    get hex(): string;
    shade(amount: number): Luum;
    tint(amount: number): Luum;
    amp(amount: number): Luum;
    mute(amount: number): Luum;
}

interface Refinement<A, B extends A> {
    (a: A): a is B;
}

type Applicator<X, Y> = (next: Modifier<X> | X) => Modifier<Y>;
type Modifier<T> = (thing: T) => T;
type OneOrMany<T> = T | T[];

declare const setHue: LuumApplicator<number>;
declare const trine: (value: number) => Modifier<LuumSpec>;
declare const tetra: (value: number) => Modifier<LuumSpec>;
declare const splitBy: (value: number) => Modifier<LuumSpec>;

declare const setSat: LuumApplicator<number>;
declare const ampBy: (value: number) => Modifier<LuumSpec>;
declare const muteBy: (value: number) => Modifier<LuumSpec>;
declare const amp: (color: LuumSpec, value: number) => LuumSpec;
declare const mute: (color: LuumSpec, value: number) => LuumSpec;

declare const resetColor: (color: LuumSpec) => LuumSpec;
declare const setLum: LuumApplicator<number>;
declare const tintBy: (tintAmount: number) => Modifier<LuumSpec>;
declare const shadeBy: (shadeAmount: number) => Modifier<LuumSpec>;
declare const tint: (color: LuumSpec, tintAmount: number) => LuumSpec;
declare const shade: (color: LuumSpec, shadeAmount: number) => LuumSpec;

declare const contrastMax: Modifier<LuumSpec>;
declare const contrastStrong: Modifier<LuumSpec>;
declare const contrastSoft: Modifier<LuumSpec>;
declare const offset: (offsetAmount: number) => Modifier<HSL>;

type LuumApplicator<X> = Applicator<X, LuumSpec>;

declare const CSS_PSEUDO_CLASSES: readonly [":active", ":checked", ":disabled", ":enabled", ":focus", ":hover", ":indeterminate", ":visited"];
type CssPseudoClass = typeof CSS_PSEUDO_CLASSES[number];
declare const isCssPseudoClass: (s: unknown) => s is ":active" | ":checked" | ":disabled" | ":enabled" | ":focus" | ":hover" | ":indeterminate" | ":visited";
declare const CSS_COLOR_PROPERTY_KEYS: readonly ["background-color", "background", "border-bottom-color", "border-color", "border-left-color", "border-right-color", "border-top-color", "border", "box-shadow", "caret-color", "color", "column-rule-color", "column-rule", "filter", "opacity", "outline-color", "outline", "text-decoration-color", "text-decoration", "text-shadow", "fill", "stroke"];
type CssVariable = `--${string}`;
type CssColorPropertyKey = CssVariable | typeof CSS_COLOR_PROPERTY_KEYS[number];
declare const isCssColorPropertyKey: (input: unknown) => input is CssColorPropertyKey;
declare const isFilterPoint: (input: unknown) => input is FilterPoint;
declare const isFilter: (input: unknown) => input is Filter;
declare const maybe: <T>(validate: Refinement<unknown, T>) => (input: unknown) => input is T | undefined;
declare const isLuumSpec: (input: unknown) => input is LuumSpec;
declare const isLuumSpecModifier: Refinement<unknown, Modifier<LuumSpec>>;
type LuumCssAttribute = [
    keys: OneOrMany<CssColorPropertyKey>,
    transformers: OneOrMany<Modifier<LuumSpec>>
];
declare const isLuumCssAttribute: (input: unknown) => input is LuumCssAttribute;
type LuumScssPseudoClassRule = [
    selectors: OneOrMany<CssPseudoClass>,
    attributes: OneOrMany<LuumCssAttribute>
];
declare const isLuumScssPseudoClassRule: (input: unknown) => input is LuumScssPseudoClassRule;
type LuumScssNestedRule = [
    selectors: OneOrMany<string>,
    attributes: OneOrMany<LuumCssAttribute>
];
declare const isLuumScssNestedRule: (input: unknown) => input is LuumScssNestedRule;
type LuumCssRule = {
    rootSelectors?: OneOrMany<string>;
    root: LuumSpec;
    attributes: OneOrMany<LuumCssAttribute>;
    filter?: Filter;
};
declare const isLuumCssRule: (input: unknown) => input is LuumCssRule;
type LuumScssRule = LuumCssRule & {
    states?: OneOrMany<LuumScssPseudoClassRule>;
    children?: OneOrMany<LuumScssNestedRule>;
};
declare const isLuumScssRule: (input: unknown) => input is LuumScssRule;
declare const RED: LuumSpec;
declare const WAYFORGE_CORE_COLOR_NAMES: readonly ["Red", "Orange", "Yellow", "Lime", "Green", "Teal", "Cyan", "Blue", "Indigo", "Violet", "Magenta", "Pink"];
type WayforgeCoreColorName = typeof WAYFORGE_CORE_COLOR_NAMES[number];
declare const WAYFORGE_CORE_COLORS: Readonly<Record<WayforgeCoreColorName, LuumSpec>>;
declare const PAINT_MY_WAGON_RED: LuumScssRule;
declare const luumToCss: (rule: LuumCssRule) => string;

declare const hueFromChannels: ({ R, G, B }: ChannelObject) => number;

declare const lumFromChannels: ({ R, G, B }: ChannelObject) => number;

declare const _default$1: (hue: Degree, filter: Filter) => number;

declare const satFromChannels: ({ R, G, B }: ChannelObject) => number;

declare const _default: (hue: number) => number;

type Interpolate = (args: {
    completionRatio: Fraction;
    range: [to: number, from: number];
}) => number;
declare const interpolate: Interpolate;

type Degree = number;
type OutOf255 = number;
type Fraction = number;
type Hex = string;
type Range = [min: number, max: number];
type ChannelObject = {
    R: number;
    G: number;
    B: number;
};
type ChannelArray = [r: number, g: number, b: number];
type FilterPoint = {
    hue: number;
    sat: number;
};
type Filter = FilterPoint[];
type HSL = {
    hue: number;
    sat: number;
    lum: number;
};
interface LuumSpec extends HSL {
    prefer: `lum` | `sat`;
}
type LuumFix = {
    sat: number;
    lum: number;
};
type LuumLimit = {
    sat: Range;
    lum: Range;
};

export { CHANNEL_SPECIFIC_LUM, CMYK, CSS_COLOR_PROPERTY_KEYS, CSS_PSEUDO_CLASSES, ChannelArray, ChannelObject, CssColorPropertyKey, CssPseudoClass, CssVariable, Degree, Filter, FilterPoint, Fraction, HSL, HUE_STRUCTURES, Hex, Luum, LuumApplicator, LuumCssAttribute, LuumCssRule, LuumFix, LuumLimit, LuumScssNestedRule, LuumScssPseudoClassRule, LuumScssRule, LuumSpec, OutOf255, PAINT_MY_WAGON_RED, RED, Range, UI, WAYFORGE_CORE_COLORS, WAYFORGE_CORE_COLOR_NAMES, WayforgeCoreColorName, amp, ampBy, channelsToSpec, contrastMax, contrastSoft, contrastStrong, _default$3 as hexToChannels, hexToSpec, hueFromChannels, _default$2 as hueToRelativeChannels, identifyHue, interpolate, isCssColorPropertyKey, isCssPseudoClass, isFilter, isFilterPoint, isLuumCssAttribute, isLuumCssRule, isLuumScssNestedRule, isLuumScssPseudoClassRule, isLuumScssRule, isLuumSpec, isLuumSpecModifier, lumFromChannels, luumToCss, _default$1 as maxSatForHueInFilter, maybe, mute, muteBy, normalizeHex, offset, palettes, resetColor, satFromChannels, setHue, setLum, setSat, shade, shadeBy, specToHex, specToHexFixLimit, _default as specificLumFromHue, splitBy, tetra, tint, tintBy, trifactory, trine, unfiltered };
